function [varargout]=micaProcdir(funstr, nproc, varargin)
%micaProc:  Parallellization function.
%Inputs:    nproc    - The number of processes you'd like to run
%	    funstr   - The function to evaluate (multiple times), e.g. 'eig'
%           P1,P2,.. - Input parameters to function given by funstr
%Output:    Q1,Q2,.. - The multidimensional outputs of the function
%Call:      [Q1,Q2,...]=paralize(funstr,P1,P2,...)
%Option:    If micaProc is called with the single argument funstr='kill', 
%           all server processes related to current directory are terminated.
%           If the argument is 'hosts' then server hosts will be listed
%Use:       Prior to running micaProc, the Server function micaServe has to be
%           started on the computers that are going to serve micaProc

%Copyright: Thomas Abrahamsson, Chalmers University of Technology, Sweden
%Written: Dec 30, 1998
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adapted for use by the Massachusetts General Hospital, Biostatistics Center
% Jacob W. Schneider
% 8/5/2003
% and 
% Peter G. Lazar
% 3/10/2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%if nargin>2 & ~exist(funstr,'file'),error(['Function ' funstr ' doesn''t seam to exist']);end
try
	if exist('lock','file'),delete('lock');end   
catch
end
if nargin==1,
   if strcmp(deblank(lower(funstr)),'kill')
      fclose(fopen('w:\killproc','w'));
      return
   elseif strcmp(deblank(lower(funstr)),'hosts')
      if exist('w:\hostnames','file'),delete('w:\hostnames');end
      fclose(fopen('w:\givename','w'));
      disp('Wait 20 secs while I''m collecting info ...')
      disp(' ')
      pause(20)
      disp('========================================================================')
      disp('HOST and computer type (with MATLAB 5.2 LU and Sparse benchmarks)')
      disp('------------------------------------------------------------------------')
      if exist('w:\hostnames','file')
         type('w:\hostnames');
         delete('w:\hostnames');
      else
         disp('No host seams to be awake.')
      end
      disp('------------------------------------------------------------------------')
      disp('If same host name occurs more than once, the same host run many sessions')
      disp('========================================================================')
      disp(' ');disp(' ');disp(' ');
      delete('w:\givename');
      return
   end
end

if (nargin==2 && strcmp(funstr,'setenv'))
	disp('Broadcasting variables. Returning in 20 secs');
	eval(['save w:\setenv' ' nproc']);
	pause(20);
	delete('w:\setenv.mat');
	return
end

% --------------------------------------------------------------------------------
%                                               Unpack the varargin cell structure
%                                               
%                                               ----------------------------------


%% m_in = number of parameters of function funstr
[n,m_in]=size(varargin);

%% m_out = number of output parameters [i.e. x = fun(arg), x is output parameter;
m_out=nargout;

%% arginstr = all variables passed-in as parameters for funstr
arginstr=[];savestr=[];


for I=1:m_in

  %% Get variables from parameter list and assign them P_some#
  eval(['P' int2str(I) '=varargin{' int2str(I) '};']);

  %% put them in the argument string
  arginstr=[arginstr 'P' int2str(I) ','];

  %% put them in the string to be passed to the Slave Nodes
  savestr=[savestr 'P' int2str(I) ' '];

end


%% argoutstr = all output parameters
argoutstr=[];

%% loadstr = all parameters for Slave Node to load upon computation 
loadstr=[];

for I=1:m_out

  %% build argoutstr and loadstr
  argoutstr=[argoutstr 'Q' int2str(I) ','];
  loadstr=[loadstr 'Q' int2str(I) ' '];

end

%% cut both strings by one (eliminates useless empty strings at end)
arginstr=arginstr(1:length(arginstr)-1);   
argoutstr=argoutstr(1:length(argoutstr)-1);   



if isempty(argoutstr) %% just perform the function

   whattodo=[funstr '(' lower(arginstr) ');'];

else %% perform the function and output to proper variables

   whattodo=['[' lower(argoutstr) ']=' funstr '(' lower(arginstr) ');'];

end

%% turn loadstr to a lower-case string
loadwhat=lower(loadstr);

%%%%
% --------------------------------------------------------------------------------
%                                                                Save data to disk
%                                                                -----------------


disp('');


fprintf(1,'Initiating process: ');


%%%%
% This loop sends-out the .mat process files that the Slave Nodes pick-up
% Editing this chunk of code will alter what data is sent to the Slave Nodes
%%%%
for I=1:nproc

  %% print-out progress of process distribution
  if I==nproc,fprintf(1,[int2str(I) '.']);else,fprintf(1,[int2str(I) ',']);end


  for II=1:m_in

	  eval(['p' int2str(II) '=P' int2str(II) ';']);

  end

  %% create .mat save file for Slave Nodes to pick-up
  eval(['process_no=' int2str(I) ';'])
  eval(['save w:\processin_' getenv('username') '' int2str(I) ' ' lower(savestr)  'process_no whattodo loadwhat']);

end

fprintf(1,'\n');

% --------------------------------------------------------------------------------
%                                                 Retreive the results and pack
%                                                 varargout
%                                                 --------------------------------


%% The number of cases is dependent upon the number of processes desired
cases=1:nproc;
caseread=0;
errormsgfile = strcat('w:\errormsgprocessin_',getenv('username'),'*','.mat');
filename = strcat('w:\processout_*processin_',getenv('username'),'*','.mat');

fprintf(1,'Reading data of #:  ');
while ~isempty(cases)
   files=dir(errormsgfile);
   if ~isempty(files)
      file=files(1,1).name(:)';
      eval(['type w:\' file]);
      eval(['delete w:\' file]);
      delete(filename);
      error('Paralize terminates');
   end
   files=dir(filename);
   if ~isempty(files)
      file=files(1,1).name(:)';
      istat=0;
      eval(['load w:\' file],'istat=1;')
      if istat~=1
        fprintf(1,[int2str(process_no) ' ']);           
       eval(['delete w:\' file]);


	%%%%
	% This loop controls the format of the output
	%%%%
        for II=1:m_out
          eval(['Q' int2str(II) '{' int2str(process_no) '}=q' int2str(II) ';'],'keyboard')
        end


%%        for II=1:m_out
%%          eval(['Q' int2str(II) '(' int2str(process_no) ',:)=q' int2str(II) ';'],'keyboard')
%%        end
	
        cases(find(cases==process_no))=[];caseread=[caseread, process_no];
      end
   end
end
fprintf(1,'\n');
      
delete(filename);

for I=1:m_out
   eval(['varargout{' int2str(I) '}=Q' int2str(I) ';'])
end
